export class PhysicsSolver {
  tasks: SolverTask[] = [];
  maxTickStep?: number;

  addTask(c: SolverTask) {
    c.solver = this;
    this.tasks.push(c);
    this.tasks.sort((a, b) => a.priority - b.priority);
  }
  removeTask(c: SolverTask) {
    this.tasks = this.tasks.filter((c1) => c1 !== c);
  }

  cleanupTasks() {
    this.tasks = this.tasks.filter((c) => c.active);
  }

  solve(dt: number) {
    dt = Math.min(dt, this.maxTickStep ?? Infinity);
    for (const c of this.tasks) {
      for (let i = 0; i < c.iterations; i++) {
        if (!c.apply(dt)) break;
      }
    }
  }
}

export abstract class SolverTask {
  iterations: number = 10;
  solver?: PhysicsSolver;
  priority: number = 0;

  active = true;

  abstract apply(dt: number): boolean;
  remove() {
    this.solver?.removeTask(this);
  }
}
