import { Vector } from "@bearmetal/doodler";
import { getContextItem } from "../../lib/context.ts";
import { Debuggable } from "../../lib/debuggable.ts";

export class Bogie extends Debuggable {
  private pos: Vector = new Vector(0, 0);
  private prevPos: Vector = this.pos.copy();
  public dir: Vector = new Vector(0, 0);
  public velocity: Vector = new Vector(0, 0);
  private acceleration: Vector = new Vector(0, 0);
  private damping: number = 0.999;

  constructor(
    public leadingOffset: number,
    public trailingOffset: number,
    public mass: number = 1,
  ) {
    super("bogies");
  }

  get position() {
    return this.pos;
  }
  set position(v: Vector) {
    this.pos.set(v);
  }

  applyForce(force: Vector) {
    this.acceleration.add(force.copy().div(this.mass));
  }

  update(dt: number) {
    // Compute velocity from previous position
    this.velocity = this.pos.copy().sub(this.prevPos);

    // Apply friction to the velocity instead of velocity storage
    this.velocity.mult(this.damping);

    // Apply Verlet integration with forces
    const temp = this.pos.copy();
    this.pos.add(this.velocity.add(this.acceleration.copy().mult(dt * dt)));
    this.prevPos = temp; // Store previous position
    this.acceleration.set(0, 0); // Reset acceleration after update
  }

  draw() {}
  debugDraw() {
    const d = getContextItem("doodler");
    d.deferDrawing(() => {
      d.dot(this.pos, {
        color: this instanceof Driver ? "red" : "white",
        weight: 3,
      });
    });
  }
}

export class Driver extends Bogie {
  public drivingForce: number = 0;

  override update(dt: number): void {
    const force = this.dir.copy();
    force.setMag(this.drivingForce);
    this.applyForce(force);
    super.update(dt);
  }
  updateDirection(dir: Vector) {
    if (this.dir.dot(dir) < -0.5) {
      // If new dir is nearly opposite, blend gradually
      this.dir = this.dir.mult(0.9).add(dir.mult(0.1)).normalize();
    } else {
      this.dir = dir;
    }
  }
}
